#!/usr/bin/env python

"""
This example shows how sending a single message works.
"""

import can


def send_one():
    """Sends a single message."""

    # this uses the default configuration (for example from the config file)
    # see https://python-can.readthedocs.io/en/stable/configuration.html
    with can.Bus() as bus:
        # Using specific buses works similar:
        # bus = can.Bus(interface='socketcan', channel='vcan0', bitrate=250000)
        # bus = can.Bus(interface='pcan', channel='PCAN_USBBUS1', bitrate=250000)
        # bus = can.Bus(interface='ixxat', channel=0, bitrate=250000)
        # bus = can.Bus(interface='vector', app_name='CANalyzer', channel=0, bitrate=250000)
        # ...

        msg = can.Message(
            arbitration_id=0xC0FFEE, data=[0, 25, 0, 1, 3, 1, 4, 1], is_extended_id=True
        )

        try:
            bus.send(msg)
            print(f"Message sent on {bus.channel_info}")
        except can.CanError:
            print("Message NOT sent")


if __name__ == "__main__":
    send_one()



#!/usr/bin/env python

"""
This demo creates multiple processes of producers to spam a socketcan bus.
"""

from concurrent.futures import ProcessPoolExecutor
from time import sleep

import can


def producer(thread_id: int, message_count: int = 16) -> None:
    """Spam the bus with messages including the data id.

    :param thread_id: the id of the thread/process
    :param message_count: the number of messages that shall be sent
    """

    # this uses the default configuration (for example from environment variables, or a
    # config file) see https://python-can.readthedocs.io/en/stable/configuration.html
    with can.Bus() as bus:
        for i in range(message_count):
            msg = can.Message(
                arbitration_id=0x0CF02200 + thread_id,
                data=[thread_id, i, 0, 1, 3, 1, 4, 1],
            )
            bus.send(msg)
        sleep(1.0)

    print(f"Producer #{thread_id} finished sending {message_count} messages")


if __name__ == "__main__":
    with ProcessPoolExecutor() as executor:
        executor.map(producer, range(5))



#!/usr/bin/env python

"""
This module tests the functions inside of bridge.py
"""

import random
import string
import sys
import threading
import time
from time import sleep as real_sleep
import unittest.mock

import can
import can.bridge
from can.interfaces import virtual

from .message_helper import ComparingMessagesTestCase


class TestBridgeScriptModule(unittest.TestCase, ComparingMessagesTestCase):

    TIMEOUT = 3.0

    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)
        ComparingMessagesTestCase.__init__(
            self,
            allowed_timestamp_delta=None,
            preserves_channel=False,
        )

    def setUp(self) -> None:
        self.stop_event = threading.Event()

        self.channel1 = "".join(random.choices(string.ascii_letters, k=8))
        self.channel2 = "".join(random.choices(string.ascii_letters, k=8))

        self.cli_args = [
            "--bus1-interface",
            "virtual",
            "--bus1-channel",
            self.channel1,
            "--bus2-interface",
            "virtual",
            "--bus2-channel",
            self.channel2,
        ]

        self.testmsg = can.Message(
            arbitration_id=0xC0FFEE, data=[0, 25, 0, 1, 3, 1, 4, 1], is_extended_id=True
        )

    def fake_sleep(self, duration):
        """A fake replacement for time.sleep that checks periodically
        whether self.stop_event is set, and raises KeyboardInterrupt
        if so.

        This allows tests to simulate an interrupt (like Ctrl+C)
        during long sleeps, in a controlled and responsive way.
        """
        interval = 0.05  # Small interval for responsiveness
        t_wakeup = time.perf_counter() + duration
        while time.perf_counter() < t_wakeup:
            if self.stop_event.is_set():
                raise KeyboardInterrupt("Simulated interrupt from fake_sleep")
            real_sleep(interval)

    def test_bridge(self):
        with (
            unittest.mock.patch("can.bridge.time.sleep", new=self.fake_sleep),
            unittest.mock.patch("can.bridge.sys.argv", [sys.argv[0], *self.cli_args]),
        ):
            # start script
            thread = threading.Thread(target=can.bridge.main)
            thread.start()

            # wait until script instantiates virtual buses
            t0 = time.perf_counter()
            while True:
                with virtual.channels_lock:
                    if (
                        self.channel1 in virtual.channels
                        and self.channel2 in virtual.channels
                    ):
                        break
                if time.perf_counter() > t0 + 2.0:
                    raise TimeoutError("Bridge script did not create virtual buses")
                real_sleep(0.2)

            # create buses with the same channels as in scripts
            with (
                can.interfaces.virtual.VirtualBus(self.channel1) as bus1,
                can.interfaces.virtual.VirtualBus(self.channel2) as bus2,
            ):
                # send test message to bus1, it should be received on bus2
                bus1.send(self.testmsg)
                recv_msg = bus2.recv(self.TIMEOUT)
                self.assertMessageEqual(self.testmsg, recv_msg)

                # assert that both buses are empty
                self.assertIsNone(bus1.recv(0))
                self.assertIsNone(bus2.recv(0))

                # send test message to bus2, it should be received on bus1
                bus2.send(self.testmsg)
                recv_msg = bus1.recv(self.TIMEOUT)
                self.assertMessageEqual(self.testmsg, recv_msg)

                # assert that both buses are empty
                self.assertIsNone(bus1.recv(0))
                self.assertIsNone(bus2.recv(0))

            # stop the bridge script
            self.stop_event.set()
            thread.join()

            # assert that the virtual buses were closed
            with virtual.channels_lock:
                self.assertNotIn(self.channel1, virtual.channels)
                self.assertNotIn(self.channel2, virtual.channels)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import time

import can


def main():
    with can.Bus(interface="virtual", receive_own_messages=True) as bus:
        print_listener = can.Printer()
        with can.Notifier(bus, listeners=[print_listener]):
            # using Notifier as a context manager automatically calls `Notifier.stop()`
            # at the end of the `with` block
            bus.send(can.Message(arbitration_id=1, is_extended_id=True))
            bus.send(can.Message(arbitration_id=2, is_extended_id=True))
            bus.send(can.Message(arbitration_id=1, is_extended_id=False))
            time.sleep(1.0)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
import contextlib
import logging
import random
import threading
import unittest

import can
from test.config import IS_PYPY

logging.getLogger(__file__).setLevel(logging.WARNING)


# make a random bool:
def rbool():
    return random.choice([False, True])


class ControllerAreaNetworkTestCase(unittest.TestCase):
    """
    This test ensures that what messages go in to the bus is what comes out.

    Requires a can interface.

    To ensure that hardware and/or software message priority queues don't
    effect the test, messages are sent one at a time.
    """

    num_messages = 512

    # TODO check if error flags are working (don't currently appear on bus)
    error_flags = [False for _ in range(num_messages)]

    remote_flags = [rbool() for _ in range(num_messages)]
    extended_flags = [rbool() for _ in range(num_messages)]

    ids = list(range(num_messages))
    data = list(
        bytearray([random.randrange(0, 2**8 - 1) for a in range(random.randrange(9))])
        for b in range(num_messages)
    )

    def setUp(self):
        # Save all can.rc defaults
        self._can_rc = can.rc
        can.rc = {"interface": "virtual"}

    def tearDown(self):
        # Restore the defaults
        can.rc = self._can_rc

    def producer(self, channel: str):
        with can.interface.Bus(channel=channel) as client_bus:
            for i in range(self.num_messages):
                m = can.Message(
                    arbitration_id=self.ids[i],
                    is_remote_frame=self.remote_flags[i],
                    is_error_frame=self.error_flags[i],
                    is_extended_id=self.extended_flags[i],
                    data=self.data[i],
                )
                client_bus.send(m)

    def testProducer(self):
        """Verify that we can send arbitrary messages on the bus"""
        logging.debug("testing producer alone")
        self.producer(channel="testProducer")
        logging.debug("producer test complete")

    def testProducerConsumer(self):
        logging.debug("testing producer/consumer")
        read_timeout = 2.0 if IS_PYPY else 0.5
        channel = "testProducerConsumer"

        with can.interface.Bus(channel=channel, interface="virtual") as server_bus:
            t = threading.Thread(target=self.producer, args=(channel,))
            t.start()

            i = 0
            while i < self.num_messages:
                msg = server_bus.recv(timeout=read_timeout)
                self.assertIsNotNone(msg, "Didn't receive a message")

                self.assertEqual(msg.is_extended_id, self.extended_flags[i])
                if not msg.is_remote_frame:
                    self.assertEqual(msg.data, self.data[i])
                self.assertEqual(msg.arbitration_id, self.ids[i])

                self.assertEqual(msg.is_error_frame, self.error_flags[i])
                self.assertEqual(msg.is_remote_frame, self.remote_flags[i])

                i += 1
            t.join()

            with contextlib.suppress(NotImplementedError):
                server_bus.flush_tx_buffer()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import unittest.mock
from typing import cast, Optional

from serial.serialutil import SerialBase

import can.interfaces.slcan

from .config import IS_PYPY

"""
Mentioned in #1010 & #1490

> PyPy works best with pure Python applications. Whenever you use a C extension module,
> it runs much slower than in CPython. The reason is that PyPy can't optimize C extension modules since they're not fully supported.
> In addition, PyPy has to emulate reference counting for that part of the code, making it even slower.

https://realpython.com/pypy-faster-python/#it-doesnt-work-well-with-c-extensions
"""
TIMEOUT = 0.5 if IS_PYPY else 0.01  # 0.001 is the default set in slcanBus


class SerialMock(SerialBase):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

        self._input_buffer = b""
        self._output_buffer = b""

    def open(self) -> None:
        self.is_open = True

    def close(self) -> None:
        self.is_open = False
        self._input_buffer = b""
        self._output_buffer = b""

    def read(self, size: int = -1, /) -> bytes:
        if size > 0:
            data = self._input_buffer[:size]
            self._input_buffer = self._input_buffer[size:]
            return data
        return b""

    def write(self, b: bytes, /) -> Optional[int]:
        self._output_buffer = b
        if b == b"N\r":
            self.set_input_buffer(b"NA123\r")
        elif b == b"V\r":
            self.set_input_buffer(b"V1013\r")
        return len(b)

    def set_input_buffer(self, expected: bytes) -> None:
        self._input_buffer = expected

    def get_output_buffer(self) -> bytes:
        return self._output_buffer

    def reset_input_buffer(self) -> None:
        self._input_buffer = b""

    @property
    def in_waiting(self) -> int:
        return len(self._input_buffer)

    @classmethod
    def serial_for_url(cls, *args, **kwargs) -> SerialBase:
        return cls(*args, **kwargs)


class slcanTestCase(unittest.TestCase):
    @unittest.mock.patch("serial.serial_for_url", SerialMock.serial_for_url)
    def setUp(self):
        self.bus = cast(
            can.interfaces.slcan.slcanBus,
            can.Bus(
                "loop://",
                interface="slcan",
                sleep_after_open=0,
                timeout=TIMEOUT,
                bitrate=500000,
            ),
        )
        self.serial = cast(SerialMock, self.bus.serialPortOrig)
        self.serial.reset_input_buffer()

    def tearDown(self):
        self.bus.shutdown()

    def test_recv_extended(self):
        self.serial.set_input_buffer(b"T12ABCDEF2AA55\r")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNotNone(msg)
        self.assertEqual(msg.arbitration_id, 0x12ABCDEF)
        self.assertEqual(msg.is_extended_id, True)
        self.assertEqual(msg.is_remote_frame, False)
        self.assertEqual(msg.dlc, 2)
        self.assertSequenceEqual(msg.data, [0xAA, 0x55])

        # Ewert Energy Systems CANDapter specific
        self.serial.set_input_buffer(b"x12ABCDEF2AA55\r")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNotNone(msg)
        self.assertEqual(msg.arbitration_id, 0x12ABCDEF)
        self.assertEqual(msg.is_extended_id, True)
        self.assertEqual(msg.is_remote_frame, False)
        self.assertEqual(msg.dlc, 2)
        self.assertSequenceEqual(msg.data, [0xAA, 0x55])

    def test_send_extended(self):
        payload = b"T12ABCDEF2AA55\r"
        msg = can.Message(
            arbitration_id=0x12ABCDEF, is_extended_id=True, data=[0xAA, 0x55]
        )
        self.bus.send(msg)
        self.assertEqual(payload, self.serial.get_output_buffer())

        self.serial.set_input_buffer(payload)
        rx_msg = self.bus.recv(TIMEOUT)
        self.assertTrue(msg.equals(rx_msg, timestamp_delta=None))

    def test_recv_standard(self):
        self.serial.set_input_buffer(b"t4563112233\r")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNotNone(msg)
        self.assertEqual(msg.arbitration_id, 0x456)
        self.assertEqual(msg.is_extended_id, False)
        self.assertEqual(msg.is_remote_frame, False)
        self.assertEqual(msg.dlc, 3)
        self.assertSequenceEqual(msg.data, [0x11, 0x22, 0x33])

    def test_send_standard(self):
        payload = b"t4563112233\r"
        msg = can.Message(
            arbitration_id=0x456, is_extended_id=False, data=[0x11, 0x22, 0x33]
        )
        self.bus.send(msg)
        self.assertEqual(payload, self.serial.get_output_buffer())

        self.serial.set_input_buffer(payload)
        rx_msg = self.bus.recv(TIMEOUT)
        self.assertTrue(msg.equals(rx_msg, timestamp_delta=None))

    def test_recv_standard_remote(self):
        self.serial.set_input_buffer(b"r1238\r")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNotNone(msg)
        self.assertEqual(msg.arbitration_id, 0x123)
        self.assertEqual(msg.is_extended_id, False)
        self.assertEqual(msg.is_remote_frame, True)
        self.assertEqual(msg.dlc, 8)

    def test_send_standard_remote(self):
        payload = b"r1238\r"
        msg = can.Message(
            arbitration_id=0x123, is_extended_id=False, is_remote_frame=True, dlc=8
        )
        self.bus.send(msg)
        self.assertEqual(payload, self.serial.get_output_buffer())

        self.serial.set_input_buffer(payload)
        rx_msg = self.bus.recv(TIMEOUT)
        self.assertTrue(msg.equals(rx_msg, timestamp_delta=None))

    def test_recv_extended_remote(self):
        self.serial.set_input_buffer(b"R12ABCDEF6\r")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNotNone(msg)
        self.assertEqual(msg.arbitration_id, 0x12ABCDEF)
        self.assertEqual(msg.is_extended_id, True)
        self.assertEqual(msg.is_remote_frame, True)
        self.assertEqual(msg.dlc, 6)

    def test_send_extended_remote(self):
        payload = b"R12ABCDEF6\r"
        msg = can.Message(
            arbitration_id=0x12ABCDEF, is_extended_id=True, is_remote_frame=True, dlc=6
        )
        self.bus.send(msg)
        self.assertEqual(payload, self.serial.get_output_buffer())

        self.serial.set_input_buffer(payload)
        rx_msg = self.bus.recv(TIMEOUT)
        self.assertTrue(msg.equals(rx_msg, timestamp_delta=None))

    def test_recv_fd(self):
        self.serial.set_input_buffer(b"d123A303132333435363738393a3b3c3d3e3f\r")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNotNone(msg)
        self.assertEqual(msg.arbitration_id, 0x123)
        self.assertEqual(msg.is_extended_id, False)
        self.assertEqual(msg.is_remote_frame, False)
        self.assertEqual(msg.is_fd, True)
        self.assertEqual(msg.bitrate_switch, False)
        self.assertEqual(msg.dlc, 16)
        self.assertSequenceEqual(
            msg.data,
            [
                0x30,
                0x31,
                0x32,
                0x33,
                0x34,
                0x35,
                0x36,
                0x37,
                0x38,
                0x39,
                0x3A,
                0x3B,
                0x3C,
                0x3D,
                0x3E,
                0x3F,
            ],
        )

    def test_send_fd(self):
        payload = b"d123A303132333435363738393A3B3C3D3E3F\r"
        msg = can.Message(
            arbitration_id=0x123,
            is_extended_id=False,
            is_fd=True,
            data=[
                0x30,
                0x31,
                0x32,
                0x33,
                0x34,
                0x35,
                0x36,
                0x37,
                0x38,
                0x39,
                0x3A,
                0x3B,
                0x3C,
                0x3D,
                0x3E,
                0x3F,
            ],
        )
        self.bus.send(msg)
        self.assertEqual(payload, self.serial.get_output_buffer())

        self.serial.set_input_buffer(payload)
        rx_msg = self.bus.recv(TIMEOUT)
        self.assertTrue(msg.equals(rx_msg, timestamp_delta=None))

    def test_recv_fd_extended(self):
        self.serial.set_input_buffer(b"D12ABCDEFA303132333435363738393A3B3C3D3E3F\r")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNotNone(msg)
        self.assertEqual(msg.arbitration_id, 0x12ABCDEF)
        self.assertEqual(msg.is_extended_id, True)
        self.assertEqual(msg.is_remote_frame, False)
        self.assertEqual(msg.dlc, 16)
        self.assertEqual(msg.bitrate_switch, False)
        self.assertTrue(msg.is_fd)
        self.assertSequenceEqual(
            msg.data,
            [
                0x30,
                0x31,
                0x32,
                0x33,
                0x34,
                0x35,
                0x36,
                0x37,
                0x38,
                0x39,
                0x3A,
                0x3B,
                0x3C,
                0x3D,
                0x3E,
                0x3F,
            ],
        )

    def test_send_fd_extended(self):
        payload = b"D12ABCDEFA303132333435363738393A3B3C3D3E3F\r"
        msg = can.Message(
            arbitration_id=0x12ABCDEF,
            is_extended_id=True,
            is_fd=True,
            data=[
                0x30,
                0x31,
                0x32,
                0x33,
                0x34,
                0x35,
                0x36,
                0x37,
                0x38,
                0x39,
                0x3A,
                0x3B,
                0x3C,
                0x3D,
                0x3E,
                0x3F,
            ],
        )
        self.bus.send(msg)
        self.assertEqual(payload, self.serial.get_output_buffer())

        self.serial.set_input_buffer(payload)
        rx_msg = self.bus.recv(TIMEOUT)
        self.assertTrue(msg.equals(rx_msg, timestamp_delta=None))

    def test_recv_fd_brs(self):
        self.serial.set_input_buffer(b"b123A303132333435363738393a3b3c3d3e3f\r")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNotNone(msg)
        self.assertEqual(msg.arbitration_id, 0x123)
        self.assertEqual(msg.is_extended_id, False)
        self.assertEqual(msg.is_remote_frame, False)
        self.assertEqual(msg.is_fd, True)
        self.assertEqual(msg.bitrate_switch, True)
        self.assertEqual(msg.dlc, 16)
        self.assertSequenceEqual(
            msg.data,
            [
                0x30,
                0x31,
                0x32,
                0x33,
                0x34,
                0x35,
                0x36,
                0x37,
                0x38,
                0x39,
                0x3A,
                0x3B,
                0x3C,
                0x3D,
                0x3E,
                0x3F,
            ],
        )

    def test_send_fd_brs(self):
        payload = b"b123A303132333435363738393A3B3C3D3E3F\r"
        msg = can.Message(
            arbitration_id=0x123,
            is_extended_id=False,
            is_fd=True,
            bitrate_switch=True,
            data=[
                0x30,
                0x31,
                0x32,
                0x33,
                0x34,
                0x35,
                0x36,
                0x37,
                0x38,
                0x39,
                0x3A,
                0x3B,
                0x3C,
                0x3D,
                0x3E,
                0x3F,
            ],
        )
        self.bus.send(msg)
        self.assertEqual(payload, self.serial.get_output_buffer())

        self.serial.set_input_buffer(payload)
        rx_msg = self.bus.recv(TIMEOUT)
        self.assertTrue(msg.equals(rx_msg, timestamp_delta=None))

    def test_recv_fd_brs_extended(self):
        self.serial.set_input_buffer(b"B12ABCDEFA303132333435363738393A3B3C3D3E3F\r")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNotNone(msg)
        self.assertEqual(msg.arbitration_id, 0x12ABCDEF)
        self.assertEqual(msg.is_extended_id, True)
        self.assertEqual(msg.is_remote_frame, False)
        self.assertEqual(msg.dlc, 16)
        self.assertEqual(msg.bitrate_switch, True)
        self.assertTrue(msg.is_fd)
        self.assertSequenceEqual(
            msg.data,
            [
                0x30,
                0x31,
                0x32,
                0x33,
                0x34,
                0x35,
                0x36,
                0x37,
                0x38,
                0x39,
                0x3A,
                0x3B,
                0x3C,
                0x3D,
                0x3E,
                0x3F,
            ],
        )

    def test_send_fd_brs_extended(self):
        payload = b"B12ABCDEFA303132333435363738393A3B3C3D3E3F\r"
        msg = can.Message(
            arbitration_id=0x12ABCDEF,
            is_extended_id=True,
            is_fd=True,
            bitrate_switch=True,
            data=[
                0x30,
                0x31,
                0x32,
                0x33,
                0x34,
                0x35,
                0x36,
                0x37,
                0x38,
                0x39,
                0x3A,
                0x3B,
                0x3C,
                0x3D,
                0x3E,
                0x3F,
            ],
        )
        self.bus.send(msg)
        self.assertEqual(payload, self.serial.get_output_buffer())

        self.serial.set_input_buffer(payload)
        rx_msg = self.bus.recv(TIMEOUT)
        self.assertTrue(msg.equals(rx_msg, timestamp_delta=None))

    def test_partial_recv(self):
        self.serial.set_input_buffer(b"T12ABCDEF")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNone(msg)

        self.serial.set_input_buffer(b"2AA55\rT12")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNotNone(msg)
        self.assertEqual(msg.arbitration_id, 0x12ABCDEF)
        self.assertEqual(msg.is_extended_id, True)
        self.assertEqual(msg.is_remote_frame, False)
        self.assertEqual(msg.dlc, 2)
        self.assertSequenceEqual(msg.data, [0xAA, 0x55])

        msg = self.bus.recv(TIMEOUT)
        self.assertIsNone(msg)

        self.serial.set_input_buffer(b"ABCDEF2AA55\r")
        msg = self.bus.recv(TIMEOUT)
        self.assertIsNotNone(msg)

    def test_version(self):
        hw_ver, sw_ver = self.bus.get_version(0)
        self.assertEqual(b"V\r", self.serial.get_output_buffer())
        self.assertEqual(hw_ver, 10)
        self.assertEqual(sw_ver, 13)

    def test_serial_number(self):
        sn = self.bus.get_serial_number(0)
        self.assertEqual(b"N\r", self.serial.get_output_buffer())
        self.assertEqual(sn, "A123")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

"""
This module tests cyclic send tasks.
"""

import gc
import sys
import time
import traceback
import unittest
from threading import Thread
from time import sleep
from typing import List
from unittest.mock import MagicMock

import can

from .config import *
from .message_helper import ComparingMessagesTestCase


class SimpleCyclicSendTaskTest(unittest.TestCase, ComparingMessagesTestCase):
    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)
        ComparingMessagesTestCase.__init__(
            self, allowed_timestamp_delta=0.016, preserves_channel=True
        )

    @unittest.skipIf(
        IS_CI,
        "the timing sensitive behaviour cannot be reproduced reliably on a CI server",
    )
    def test_cycle_time(self):
        msg = can.Message(
            is_extended_id=False, arbitration_id=0x123, data=[0, 1, 2, 3, 4, 5, 6, 7]
        )

        with can.interface.Bus(interface="virtual") as bus1:
            with can.interface.Bus(interface="virtual") as bus2:
                # disabling the garbage collector makes the time readings more reliable
                gc.disable()

                task = bus1.send_periodic(msg, 0.01, 1)
                self.assertIsInstance(task, can.broadcastmanager.CyclicSendTaskABC)

                sleep(2)
                size = bus2.queue.qsize()
                # About 100 messages should have been transmitted
                self.assertTrue(
                    80 <= size <= 120,
                    "100 +/- 20 messages should have been transmitted. But queue contained {}".format(
                        size
                    ),
                )
                last_msg = bus2.recv()
                next_last_msg = bus2.recv()

                # we need to reenable the garbage collector again
                gc.enable()

                # Check consecutive messages are spaced properly in time and have
                # the same id/data
                self.assertMessageEqual(last_msg, next_last_msg)

                # Check the message id/data sent is the same as message received
                # Set timestamp and channel to match recv'd because we don't care
                # and they are not initialized by the can.Message constructor.
                msg.timestamp = last_msg.timestamp
                msg.channel = last_msg.channel
                self.assertMessageEqual(msg, last_msg)

    def test_removing_bus_tasks(self):
        bus = can.interface.Bus(interface="virtual")
        tasks = []
        for task_i in range(10):
            msg = can.Message(
                is_extended_id=False,
                arbitration_id=0x123,
                data=[0, 1, 2, 3, 4, 5, 6, 7],
            )
            msg.arbitration_id = task_i
            task = bus.send_periodic(msg, 0.1, 1)
            tasks.append(task)
            self.assertIsInstance(task, can.broadcastmanager.CyclicSendTaskABC)

        assert len(bus._periodic_tasks) == 10

        for task in tasks:
            # Note calling task.stop will remove the task from the Bus's internal task management list
            task.stop()

        self.join_threads([task.thread for task in tasks], 5.0)

        assert len(bus._periodic_tasks) == 0
        bus.shutdown()

    def test_managed_tasks(self):
        bus = can.interface.Bus(interface="virtual", receive_own_messages=True)
        tasks = []
        for task_i in range(3):
            msg = can.Message(
                is_extended_id=False,
                arbitration_id=0x123,
                data=[0, 1, 2, 3, 4, 5, 6, 7],
            )
            msg.arbitration_id = task_i
            task = bus.send_periodic(msg, 0.1, 10, store_task=False)
            tasks.append(task)
            self.assertIsInstance(task, can.broadcastmanager.CyclicSendTaskABC)

        assert len(bus._periodic_tasks) == 0

        # Self managed tasks should still be sending messages
        for _ in range(50):
            received_msg = bus.recv(timeout=5.0)
            assert received_msg is not None
            assert received_msg.arbitration_id in {0, 1, 2}

        for task in tasks:
            task.stop()

        self.join_threads([task.thread for task in tasks], 5.0)

        bus.shutdown()

    def test_stopping_perodic_tasks(self):
        bus = can.interface.Bus(interface="virtual")
        tasks = []
        for task_i in range(10):
            msg = can.Message(
                is_extended_id=False,
                arbitration_id=0x123,
                data=[0, 1, 2, 3, 4, 5, 6, 7],
            )
            msg.arbitration_id = task_i
            task = bus.send_periodic(msg, 0.1, 1)
            tasks.append(task)

        assert len(bus._periodic_tasks) == 10
        # stop half the tasks using the task object
        for task in tasks[::2]:
            task.stop()

        assert len(bus._periodic_tasks) == 5

        # stop the other half using the bus api
        bus.stop_all_periodic_tasks(remove_tasks=False)
        self.join_threads([task.thread for task in tasks], 5.0)

        # Tasks stopped via `stop_all_periodic_tasks` with remove_tasks=False should
        # still be associated with the bus (e.g. for restarting)
        assert len(bus._periodic_tasks) == 5

        bus.shutdown()

    def test_restart_perodic_tasks(self):
        period = 0.01
        safe_timeout = period * 5 if not IS_PYPY else 1.0
        duration = 0.3

        msg = can.Message(
            is_extended_id=False, arbitration_id=0x123, data=[0, 1, 2, 3, 4, 5, 6, 7]
        )

        def _read_all_messages(_bus: "can.interfaces.virtual.VirtualBus") -> None:
            sleep(safe_timeout)
            while not _bus.queue.empty():
                _bus.recv(timeout=period)
            sleep(safe_timeout)

        with can.ThreadSafeBus(interface="virtual", receive_own_messages=True) as bus:
            task = bus.send_periodic(msg, period)
            self.assertIsInstance(task, can.broadcastmanager.ThreadBasedCyclicSendTask)

            # Test that the task is sending messages
            sleep(safe_timeout)
            assert not bus.queue.empty(), "messages should have been transmitted"

            # Stop the task and check that messages are no longer being sent
            bus.stop_all_periodic_tasks(remove_tasks=False)
            _read_all_messages(bus)
            assert bus.queue.empty(), "messages should not have been transmitted"

            # Restart the task and check that messages are being sent again
            task.start()
            sleep(safe_timeout)
            assert not bus.queue.empty(), "messages should have been transmitted"

            # Stop the task and check that messages are no longer being sent
            bus.stop_all_periodic_tasks(remove_tasks=False)
            _read_all_messages(bus)
            assert bus.queue.empty(), "messages should not have been transmitted"

            # Restart the task with limited duration and wait until it stops
            task.duration = duration
            task.start()
            sleep(duration + safe_timeout)
            assert task.stopped
            assert time.time() > task.end_time
            assert not bus.queue.empty(), "messages should have been transmitted"
            _read_all_messages(bus)
            assert bus.queue.empty(), "messages should not have been transmitted"

            # Restart the task and check that messages are being sent again
            task.start()
            sleep(safe_timeout)
            assert not bus.queue.empty(), "messages should have been transmitted"

            # Stop all tasks and wait for the thread to exit
            bus.stop_all_periodic_tasks()
            # Avoids issues where the thread is still running when the bus is shutdown
            self.join_threads([task.thread], 5.0)

    @unittest.skipIf(IS_CI, "fails randomly when run on CI server")
    def test_thread_based_cyclic_send_task(self):
        bus = can.ThreadSafeBus(interface="virtual")
        msg = can.Message(
            is_extended_id=False, arbitration_id=0x123, data=[0, 1, 2, 3, 4, 5, 6, 7]
        )

        # good case, bus is up
        on_error_mock = MagicMock(return_value=False)
        task = can.broadcastmanager.ThreadBasedCyclicSendTask(
            bus=bus,
            lock=bus._lock_send_periodic,
            messages=msg,
            period=0.1,
            duration=3,
            on_error=on_error_mock,
        )
        sleep(1)
        on_error_mock.assert_not_called()
        task.stop()
        bus.shutdown()

        # bus has been shut down
        on_error_mock = MagicMock(return_value=False)
        task = can.broadcastmanager.ThreadBasedCyclicSendTask(
            bus=bus,
            lock=bus._lock_send_periodic,
            messages=msg,
            period=0.1,
            duration=3,
            on_error=on_error_mock,
        )
        sleep(1)
        self.assertEqual(1, on_error_mock.call_count)
        task.stop()

        # bus is still shut down, but on_error returns True
        on_error_mock = MagicMock(return_value=True)
        task = can.broadcastmanager.ThreadBasedCyclicSendTask(
            bus=bus,
            lock=bus._lock_send_periodic,
            messages=msg,
            period=0.1,
            duration=3,
            on_error=on_error_mock,
        )
        sleep(1)
        self.assertTrue(on_error_mock.call_count > 1)
        task.stop()

    def test_modifier_callback(self) -> None:
        msg_list: List[can.Message] = []

        def increment_first_byte(msg: can.Message) -> None:
            msg.data[0] = (msg.data[0] + 1) % 256

        original_msg = can.Message(
            is_extended_id=False, arbitration_id=0x123, data=[0] * 8
        )

        with can.ThreadSafeBus(interface="virtual", receive_own_messages=True) as bus:
            notifier = can.Notifier(bus=bus, listeners=[msg_list.append])
            task = bus.send_periodic(
                msgs=original_msg, period=0.001, modifier_callback=increment_first_byte
            )
            time.sleep(0.2)
            task.stop()
            notifier.stop()

        self.assertEqual(b"\x01\x00\x00\x00\x00\x00\x00\x00", bytes(msg_list[0].data))
        self.assertEqual(b"\x02\x00\x00\x00\x00\x00\x00\x00", bytes(msg_list[1].data))
        self.assertEqual(b"\x03\x00\x00\x00\x00\x00\x00\x00", bytes(msg_list[2].data))
        self.assertEqual(b"\x04\x00\x00\x00\x00\x00\x00\x00", bytes(msg_list[3].data))
        self.assertEqual(b"\x05\x00\x00\x00\x00\x00\x00\x00", bytes(msg_list[4].data))
        self.assertEqual(b"\x06\x00\x00\x00\x00\x00\x00\x00", bytes(msg_list[5].data))
        self.assertEqual(b"\x07\x00\x00\x00\x00\x00\x00\x00", bytes(msg_list[6].data))

    @staticmethod
    def join_threads(threads: List[Thread], timeout: float) -> None:
        stuck_threads: List[Thread] = []
        t0 = time.perf_counter()
        for thread in threads:
            time_left = timeout - (time.perf_counter() - t0)
            if time_left > 0.0:
                thread.join(time_left)
            if thread.is_alive():
                if platform.python_implementation() == "CPython":
                    # print thread frame to help with debugging
                    frame = sys._current_frames()[thread.ident]
                    traceback.print_stack(frame, file=sys.stderr)
                stuck_threads.append(thread)
        if stuck_threads:
            err_message = (
                f"Threads did not stop within {timeout:.1f} seconds: "
                f"[{', '.join([str(t) for t in stuck_threads])}]"
            )
            raise RuntimeError(err_message)


if __name__ == "__main__":
    unittest.main()



import argparse
import unittest
from unittest.mock import patch

from can.cli import add_bus_arguments, create_bus_from_namespace


class TestCliUtils(unittest.TestCase):
    def test_add_bus_arguments(self):
        parser = argparse.ArgumentParser()
        add_bus_arguments(parser, filter_arg=True, prefix="test")

        parsed_args = parser.parse_args(
            [
                "--test-channel",
                "0",
                "--test-interface",
                "vector",
                "--test-timing",
                "f_clock=8000000",
                "brp=4",
                "tseg1=11",
                "tseg2=4",
                "sjw=2",
                "nof_samples=3",
                "--test-filter",
                "100:7FF",
                "200~7F0",
                "--test-bus-kwargs",
                "app_name=MyApp",
                "serial=1234",
            ]
        )

        self.assertNotIn("channel", parsed_args)
        self.assertNotIn("test_bitrate", parsed_args)
        self.assertNotIn("test_data_bitrate", parsed_args)
        self.assertNotIn("test_fd", parsed_args)

        self.assertEqual(parsed_args.test_channel, "0")
        self.assertEqual(parsed_args.test_interface, "vector")
        self.assertEqual(parsed_args.test_timing.f_clock, 8000000)
        self.assertEqual(parsed_args.test_timing.brp, 4)
        self.assertEqual(parsed_args.test_timing.tseg1, 11)
        self.assertEqual(parsed_args.test_timing.tseg2, 4)
        self.assertEqual(parsed_args.test_timing.sjw, 2)
        self.assertEqual(parsed_args.test_timing.nof_samples, 3)
        self.assertEqual(len(parsed_args.test_can_filters), 2)
        self.assertEqual(parsed_args.test_can_filters[0]["can_id"], 0x100)
        self.assertEqual(parsed_args.test_can_filters[0]["can_mask"], 0x7FF)
        self.assertEqual(parsed_args.test_can_filters[1]["can_id"], 0x200 | 0x20000000)
        self.assertEqual(
            parsed_args.test_can_filters[1]["can_mask"], 0x7F0 & 0x20000000
        )
        self.assertEqual(parsed_args.test_bus_kwargs["app_name"], "MyApp")
        self.assertEqual(parsed_args.test_bus_kwargs["serial"], 1234)

    def test_add_bus_arguments_no_prefix(self):
        parser = argparse.ArgumentParser()
        add_bus_arguments(parser, filter_arg=True)

        parsed_args = parser.parse_args(
            [
                "--channel",
                "0",
                "--interface",
                "vector",
                "--timing",
                "f_clock=8000000",
                "brp=4",
                "tseg1=11",
                "tseg2=4",
                "sjw=2",
                "nof_samples=3",
                "--filter",
                "100:7FF",
                "200~7F0",
                "--bus-kwargs",
                "app_name=MyApp",
                "serial=1234",
            ]
        )

        self.assertEqual(parsed_args.channel, "0")
        self.assertEqual(parsed_args.interface, "vector")
        self.assertEqual(parsed_args.timing.f_clock, 8000000)
        self.assertEqual(parsed_args.timing.brp, 4)
        self.assertEqual(parsed_args.timing.tseg1, 11)
        self.assertEqual(parsed_args.timing.tseg2, 4)
        self.assertEqual(parsed_args.timing.sjw, 2)
        self.assertEqual(parsed_args.timing.nof_samples, 3)
        self.assertEqual(len(parsed_args.can_filters), 2)
        self.assertEqual(parsed_args.can_filters[0]["can_id"], 0x100)
        self.assertEqual(parsed_args.can_filters[0]["can_mask"], 0x7FF)
        self.assertEqual(parsed_args.can_filters[1]["can_id"], 0x200 | 0x20000000)
        self.assertEqual(parsed_args.can_filters[1]["can_mask"], 0x7F0 & 0x20000000)
        self.assertEqual(parsed_args.bus_kwargs["app_name"], "MyApp")
        self.assertEqual(parsed_args.bus_kwargs["serial"], 1234)

    @patch("can.Bus")
    def test_create_bus_from_namespace(self, mock_bus):
        namespace = argparse.Namespace(
            test_channel="vcan0",
            test_interface="virtual",
            test_bitrate=500000,
            test_data_bitrate=2000000,
            test_fd=True,
            test_can_filters=[{"can_id": 0x100, "can_mask": 0x7FF}],
            test_bus_kwargs={"app_name": "MyApp", "serial": 1234},
        )

        create_bus_from_namespace(namespace, prefix="test")

        mock_bus.assert_called_once_with(
            channel="vcan0",
            interface="virtual",
            bitrate=500000,
            data_bitrate=2000000,
            fd=True,
            can_filters=[{"can_id": 0x100, "can_mask": 0x7FF}],
            app_name="MyApp",
            serial=1234,
            single_handle=True,
        )

    @patch("can.Bus")
    def test_create_bus_from_namespace_no_prefix(self, mock_bus):
        namespace = argparse.Namespace(
            channel="vcan0",
            interface="virtual",
            bitrate=500000,
            data_bitrate=2000000,
            fd=True,
            can_filters=[{"can_id": 0x100, "can_mask": 0x7FF}],
            bus_kwargs={"app_name": "MyApp", "serial": 1234},
        )

        create_bus_from_namespace(namespace)

        mock_bus.assert_called_once_with(
            channel="vcan0",
            interface="virtual",
            bitrate=500000,
            data_bitrate=2000000,
            fd=True,
            can_filters=[{"can_id": 0x100, "can_mask": 0x7FF}],
            app_name="MyApp",
            serial=1234,
            single_handle=True,
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

"""
This example demonstrates how to send a periodic message containing
an automatically updating counter and checksum.

Expects a virtual interface:

    python3 -m examples.cyclic_checksum
"""

import logging
import time

import can

logging.basicConfig(level=logging.INFO)


def cyclic_checksum_send(bus: can.BusABC) -> None:
    """
    Sends periodic messages every 1 s with no explicit timeout.
    The message's counter and checksum is updated before each send.
    Sleeps for 10 seconds then stops the task.
    """
    message = can.Message(arbitration_id=0x78, data=[0, 1, 2, 3, 4, 5, 6, 0])
    print("Starting to send an auto-updating message every 100ms for 3 s")
    task = bus.send_periodic(msgs=message, period=0.1, modifier_callback=update_message)
    time.sleep(3)
    task.stop()
    print("stopped cyclic send")


def update_message(message: can.Message) -> None:
    counter = increment_counter(message)
    checksum = compute_xbr_checksum(message, counter)
    message.data[7] = (checksum << 4) + counter


def increment_counter(message: can.Message) -> int:
    counter = message.data[7] & 0x0F
    counter += 1
    counter %= 16

    return counter


def compute_xbr_checksum(message: can.Message, counter: int) -> int:
    """
    Computes an XBR checksum as per SAE J1939 SPN 3188.
    """
    checksum = sum(message.data[:7])
    checksum += sum(message.arbitration_id.to_bytes(length=4, byteorder="big"))
    checksum += counter & 0x0F
    xbr_checksum = ((checksum >> 4) + checksum) & 0x0F

    return xbr_checksum


if __name__ == "__main__":
    with can.Bus(channel=0, interface="virtual", receive_own_messages=True) as _bus:
        with can.Notifier(bus=_bus, listeners=[print]):
            cyclic_checksum_send(_bus)



#!/usr/bin/env python

"""
This shows how message filtering works.
"""

import time

import can


def main():
    """Send some messages to itself and apply filtering."""
    with can.Bus(interface="virtual", receive_own_messages=True) as bus:
        can_filters = [{"can_id": 1, "can_mask": 0xF, "extended": True}]
        bus.set_filters(can_filters)

        # print all incoming messages, which includes the ones sent,
        # since we set receive_own_messages to True
        # assign to some variable so it does not garbage collected
        with can.Notifier(bus, [can.Printer()]):  # pylint: disable=unused-variable
            bus.send(can.Message(arbitration_id=1, is_extended_id=True))
            bus.send(can.Message(arbitration_id=2, is_extended_id=True))
            bus.send(can.Message(arbitration_id=1, is_extended_id=False))
            time.sleep(1.0)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Shows how to receive messages via polling.
"""

import can
from can.bus import BusState


def receive_all():
    """Receives all messages and prints them to the console until Ctrl+C is pressed."""

    # this uses the default configuration (for example from environment variables, or a
    # config file) see https://python-can.readthedocs.io/en/stable/configuration.html
    with can.Bus() as bus:
        # set to read-only, only supported on some interfaces
        try:
            bus.state = BusState.PASSIVE
        except NotImplementedError:
            pass

        try:
            while True:
                msg = bus.recv(1)
                if msg is not None:
                    print(msg)

        except KeyboardInterrupt:
            pass  # exit normally


if __name__ == "__main__":
    receive_all()



#!/usr/bin/env python

"""
Test rotating loggers
"""

import os
from pathlib import Path
from typing import cast
from unittest.mock import Mock

import can
from can.io.generic import FileIOMessageWriter
from can.typechecking import StringPathLike

from .data.example_data import generate_message


class TestBaseRotatingLogger:
    @staticmethod
    def _get_instance(file: StringPathLike) -> can.io.BaseRotatingLogger:
        class SubClass(can.io.BaseRotatingLogger):
            """Subclass that implements abstract methods for testing."""

            _supported_formats = {".asc", ".blf", ".csv", ".log", ".txt"}

            def __init__(self, file: StringPathLike, **kwargs) -> None:
                super().__init__(**kwargs)
                suffix = Path(file).suffix.lower()
                if suffix not in self._supported_formats:
                    raise ValueError(f"Unsupported file format: {suffix}")
                self._writer = can.Logger(filename=file)

            @property
            def writer(self) -> FileIOMessageWriter:
                return cast(FileIOMessageWriter, self._writer)

            def should_rollover(self, msg: can.Message) -> bool:
                return False

            def do_rollover(self): ...

        return SubClass(file=file)

    def test_import(self):
        assert hasattr(can.io, "BaseRotatingLogger")

    def test_attributes(self):
        assert issubclass(can.io.BaseRotatingLogger, can.Listener)
        assert hasattr(can.io.BaseRotatingLogger, "namer")
        assert hasattr(can.io.BaseRotatingLogger, "rotator")
        assert hasattr(can.io.BaseRotatingLogger, "rollover_count")
        assert hasattr(can.io.BaseRotatingLogger, "writer")
        assert hasattr(can.io.BaseRotatingLogger, "rotation_filename")
        assert hasattr(can.io.BaseRotatingLogger, "rotate")
        assert hasattr(can.io.BaseRotatingLogger, "on_message_received")
        assert hasattr(can.io.BaseRotatingLogger, "stop")
        assert hasattr(can.io.BaseRotatingLogger, "should_rollover")
        assert hasattr(can.io.BaseRotatingLogger, "do_rollover")

    def test_get_new_writer(self, tmp_path):
        with self._get_instance(tmp_path / "file.ASC") as logger_instance:
            assert isinstance(logger_instance.writer, can.ASCWriter)

        with self._get_instance(tmp_path / "file.BLF") as logger_instance:
            assert isinstance(logger_instance.writer, can.BLFWriter)

        with self._get_instance(tmp_path / "file.CSV") as logger_instance:
            assert isinstance(logger_instance.writer, can.CSVWriter)

        with self._get_instance(tmp_path / "file.LOG") as logger_instance:
            assert isinstance(logger_instance.writer, can.CanutilsLogWriter)

        with self._get_instance(tmp_path / "file.TXT") as logger_instance:
            assert isinstance(logger_instance.writer, can.Printer)

    def test_rotation_filename(self, tmp_path):
        with self._get_instance(tmp_path / "__unused.txt") as logger_instance:
            default_name = "default"
            assert logger_instance.rotation_filename(default_name) == "default"

            logger_instance.namer = lambda x: x + "_by_namer"
            assert logger_instance.rotation_filename(default_name) == "default_by_namer"

    def test_rotate_without_rotator(self, tmp_path):
        source = str(tmp_path / "source.txt")
        dest = str(tmp_path / "dest.txt")

        assert os.path.exists(source) is False
        assert os.path.exists(dest) is False

        with self._get_instance(source) as logger_instance:
            # use context manager to create `source` file and close it
            pass

        assert os.path.exists(source) is True
        assert os.path.exists(dest) is False

        logger_instance.rotate(source, dest)

        assert os.path.exists(source) is False
        assert os.path.exists(dest) is True

    def test_rotate_with_rotator(self, tmp_path):
        source = str(tmp_path / "source.txt")
        dest = str(tmp_path / "dest.txt")

        assert os.path.exists(source) is False
        assert os.path.exists(dest) is False

        with self._get_instance(source) as logger_instance:
            # use context manager to create `source` file and close it
            pass

        rotator_func = Mock()
        logger_instance.rotator = rotator_func
        logger_instance._writer = logger_instance._get_new_writer(source)
        logger_instance.stop()

        assert os.path.exists(source) is True
        assert os.path.exists(dest) is False

        logger_instance.rotate(source, dest)
        rotator_func.assert_called_with(source, dest)

        # assert that no rotation was performed since rotator_func
        # does not do anything
        assert os.path.exists(source) is True
        assert os.path.exists(dest) is False

    def test_stop(self, tmp_path):
        """Test if stop() method of writer is called."""
        with self._get_instance(tmp_path / "file.ASC") as logger_instance:
            # replace stop method of writer with Mock
            mock_stop = Mock(side_effect=logger_instance.writer.stop)
            logger_instance.writer.stop = mock_stop

            logger_instance.stop()
            mock_stop.assert_called()

    def test_on_message_received(self, tmp_path):
        with self._get_instance(tmp_path / "file.ASC") as logger_instance:
            # Test without rollover
            should_rollover = Mock(return_value=False)
            do_rollover = Mock()
            writers_on_message_received = Mock()

            logger_instance.should_rollover = should_rollover
            logger_instance.do_rollover = do_rollover
            logger_instance.writer.on_message_received = writers_on_message_received

            msg = generate_message(0x123)
            logger_instance.on_message_received(msg)

            should_rollover.assert_called_with(msg)
            do_rollover.assert_not_called()
            writers_on_message_received.assert_called_with(msg)

            # Test with rollover
            should_rollover = Mock(return_value=True)
            do_rollover = Mock()
            writers_on_message_received = Mock()

            logger_instance.should_rollover = should_rollover
            logger_instance.do_rollover = do_rollover
            logger_instance.writer.on_message_received = writers_on_message_received

            msg = generate_message(0x123)
            logger_instance.on_message_received(msg)

            should_rollover.assert_called_with(msg)
            do_rollover.assert_called()
            writers_on_message_received.assert_called_with(msg)

    def test_issue_1792(self, tmp_path):
        filepath = tmp_path / "2017_Jeep_Grand_Cherokee_3.6L_V6.log"
        with self._get_instance(filepath) as logger_instance:
            assert isinstance(logger_instance.writer, can.CanutilsLogWriter)


class TestSizedRotatingLogger:
    def test_import(self):
        assert hasattr(can.io, "SizedRotatingLogger")
        assert hasattr(can, "SizedRotatingLogger")

    def test_attributes(self):
        assert issubclass(can.SizedRotatingLogger, can.io.BaseRotatingLogger)
        assert hasattr(can.SizedRotatingLogger, "namer")
        assert hasattr(can.SizedRotatingLogger, "rotator")
        assert hasattr(can.SizedRotatingLogger, "should_rollover")
        assert hasattr(can.SizedRotatingLogger, "do_rollover")

    def test_create_instance(self, tmp_path):
        base_filename = "mylogfile.ASC"
        max_bytes = 512

        with can.SizedRotatingLogger(
            base_filename=tmp_path / base_filename, max_bytes=max_bytes
        ) as logger_instance:
            assert Path(logger_instance.base_filename).name == base_filename
            assert logger_instance.max_bytes == max_bytes
            assert logger_instance.rollover_count == 0
            assert isinstance(logger_instance.writer, can.ASCWriter)

    def test_should_rollover(self, tmp_path):
        base_filename = "mylogfile.ASC"
        max_bytes = 512

        with can.SizedRotatingLogger(
            base_filename=tmp_path / base_filename, max_bytes=max_bytes
        ) as logger_instance:
            msg = generate_message(0x123)
            do_rollover = Mock()
            logger_instance.do_rollover = do_rollover

            logger_instance.writer.file.tell = Mock(return_value=511)
            assert logger_instance.should_rollover(msg) is False
            logger_instance.on_message_received(msg)
            do_rollover.assert_not_called()

            logger_instance.writer.file.tell = Mock(return_value=512)
            assert logger_instance.should_rollover(msg) is True
            logger_instance.on_message_received(msg)
            do_rollover.assert_called()

    def test_logfile_size(self, tmp_path):
        base_filename = "mylogfile.ASC"
        max_bytes = 1024
        msg = generate_message(0x123)

        with can.SizedRotatingLogger(
            base_filename=tmp_path / base_filename, max_bytes=max_bytes
        ) as logger_instance:
            for _ in range(128):
                logger_instance.on_message_received(msg)

            for file_path in os.listdir(tmp_path):
                assert os.path.getsize(tmp_path / file_path) <= 1100

    def test_logfile_size_context_manager(self, tmp_path):
        base_filename = "mylogfile.ASC"
        max_bytes = 1024
        msg = generate_message(0x123)

        with can.SizedRotatingLogger(
            base_filename=tmp_path / base_filename, max_bytes=max_bytes
        ) as logger_instance:
            for _ in range(128):
                logger_instance.on_message_received(msg)

            for file_path in os.listdir(tmp_path):
                assert os.path.getsize(os.path.join(tmp_path, file_path)) <= 1100



#!/usr/bin/env python

"""
Use this to convert .can/.asc files to .log files.
Can be easily adapted for all sorts of files.

Usage: python3 simple_log_convert.py sourceLog.asc targetLog.log
"""

import sys

import can


def main():
    """The transcoder"""

    with can.LogReader(sys.argv[1]) as reader:
        with can.Logger(sys.argv[2]) as writer:
            for msg in reader:
                writer.on_message_received(msg)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This example exercises the periodic task's multiple message sending capabilities

Expects a vcan0 interface:

    python3 -m examples.cyclic_multiple

"""

import logging
import time

import can

logging.basicConfig(level=logging.INFO)


def cyclic_multiple_send(bus):
    """
    Sends periodic messages every 1 s with no explicit timeout
    Sleeps for 10 seconds then stops the task.
    """
    print("Starting to send a message every 1 s for 10 s")
    messages = []

    messages.append(
        can.Message(
            arbitration_id=0x401,
            data=[0x11, 0x11, 0x11, 0x11, 0x11, 0x11],
            is_extended_id=False,
        )
    )
    messages.append(
        can.Message(
            arbitration_id=0x401,
            data=[0x22, 0x22, 0x22, 0x22, 0x22, 0x22],
            is_extended_id=False,
        )
    )
    messages.append(
        can.Message(
            arbitration_id=0x401,
            data=[0x33, 0x33, 0x33, 0x33, 0x33, 0x33],
            is_extended_id=False,
        )
    )
    messages.append(
        can.Message(
            arbitration_id=0x401,
            data=[0x44, 0x44, 0x44, 0x44, 0x44, 0x44],
            is_extended_id=False,
        )
    )
    messages.append(
        can.Message(
            arbitration_id=0x401,
            data=[0x55, 0x55, 0x55, 0x55, 0x55, 0x55],
            is_extended_id=False,
        )
    )
    task = bus.send_periodic(messages, 1)
    assert isinstance(task, can.CyclicSendTaskABC)
    time.sleep(10)
    task.stop()
    print("stopped cyclic send")


def cyclic_multiple_send_modify(bus):
    """
    Sends initial set of 3 Messages containing Odd data sent every 1 s with
    no explicit timeout. Sleeps for 8 s.

    Then the set is updated to 3 Messages containing Even data.
    Sleeps for 10 s.
    """
    messages_odd = []
    messages_odd.append(
        can.Message(
            arbitration_id=0x401,
            data=[0x11, 0x11, 0x11, 0x11, 0x11, 0x11],
            is_extended_id=False,
        )
    )
    messages_odd.append(
        can.Message(
            arbitration_id=0x401,
            data=[0x33, 0x33, 0x33, 0x33, 0x33, 0x33],
            is_extended_id=False,
        )
    )
    messages_odd.append(
        can.Message(
            arbitration_id=0x401,
            data=[0x55, 0x55, 0x55, 0x55, 0x55, 0x55],
            is_extended_id=False,
        )
    )
    messages_even = []
    messages_even.append(
        can.Message(
            arbitration_id=0x401,
            data=[0x22, 0x22, 0x22, 0x22, 0x22, 0x22],
            is_extended_id=False,
        )
    )
    messages_even.append(
        can.Message(
            arbitration_id=0x401,
            data=[0x44, 0x44, 0x44, 0x44, 0x44, 0x44],
            is_extended_id=False,
        )
    )
    messages_even.append(
        can.Message(
            arbitration_id=0x401,
            data=[0x66, 0x66, 0x66, 0x66, 0x66, 0x66],
            is_extended_id=False,
        )
    )
    print("Starting to send a message with odd every 1 s for 8 s with odd data")
    task = bus.send_periodic(messages_odd, 1)
    assert isinstance(task, can.CyclicSendTaskABC)
    time.sleep(8)
    print("Starting to send a message with even data every 1 s for 10 s with even data")
    task.modify_data(messages_even)
    time.sleep(10)
    print("stopped cyclic modify send")


if __name__ == "__main__":
    for interface, channel in [("socketcan", "vcan0")]:
        print(f"Carrying out cyclic multiple tests with {interface} interface")

        with can.Bus(interface=interface, channel=channel, bitrate=500000) as BUS:
            cyclic_multiple_send(BUS)
            cyclic_multiple_send_modify(BUS)

    time.sleep(2)



#!/usr/bin/env python

import asyncio
import time
import unittest

import can


class NotifierTest(unittest.TestCase):
    def test_single_bus(self):
        with can.Bus("test", interface="virtual", receive_own_messages=True) as bus:
            reader = can.BufferedReader()
            notifier = can.Notifier(bus, [reader], 0.1)
            self.assertFalse(notifier.stopped)
            msg = can.Message()
            bus.send(msg)
            self.assertIsNotNone(reader.get_message(1))
            notifier.stop()
            self.assertTrue(notifier.stopped)

    def test_multiple_bus(self):
        with can.Bus(0, interface="virtual", receive_own_messages=True) as bus1:
            with can.Bus(1, interface="virtual", receive_own_messages=True) as bus2:
                reader = can.BufferedReader()
                notifier = can.Notifier([bus1, bus2], [reader], 0.1)
                self.assertFalse(notifier.stopped)
                msg = can.Message()
                bus1.send(msg)
                time.sleep(0.1)
                bus2.send(msg)
                recv_msg = reader.get_message(1)
                self.assertIsNotNone(recv_msg)
                self.assertEqual(recv_msg.channel, 0)
                recv_msg = reader.get_message(1)
                self.assertIsNotNone(recv_msg)
                self.assertEqual(recv_msg.channel, 1)
                notifier.stop()
                self.assertTrue(notifier.stopped)

    def test_context_manager(self):
        with can.Bus("test", interface="virtual", receive_own_messages=True) as bus:
            reader = can.BufferedReader()
            with can.Notifier(bus, [reader], 0.1) as notifier:
                self.assertFalse(notifier.stopped)
                msg = can.Message()
                bus.send(msg)
                self.assertIsNotNone(reader.get_message(1))
                notifier.stop()
                self.assertTrue(notifier.stopped)

    def test_registry(self):
        with can.Bus("test", interface="virtual", receive_own_messages=True) as bus:
            reader = can.BufferedReader()
            with can.Notifier(bus, [reader], 0.1) as notifier:
                # creating a second notifier for the same bus must fail
                self.assertRaises(ValueError, can.Notifier, bus, [reader], 0.1)

                # find_instance must return the existing instance
                self.assertEqual(can.Notifier.find_instances(bus), (notifier,))

            # Notifier is stopped, find_instances() must return an empty tuple
            self.assertEqual(can.Notifier.find_instances(bus), ())

            # now the first notifier is stopped, a new notifier can be created without error:
            with can.Notifier(bus, [reader], 0.1) as notifier:
                # the next notifier call should fail again since there is an active notifier already
                self.assertRaises(ValueError, can.Notifier, bus, [reader], 0.1)

                # find_instance must return the existing instance
                self.assertEqual(can.Notifier.find_instances(bus), (notifier,))


class AsyncNotifierTest(unittest.TestCase):
    def test_asyncio_notifier(self):
        async def run_it():
            with can.Bus("test", interface="virtual", receive_own_messages=True) as bus:
                reader = can.AsyncBufferedReader()
                notifier = can.Notifier(
                    bus, [reader], 0.1, loop=asyncio.get_running_loop()
                )
                bus.send(can.Message())
                recv_msg = await asyncio.wait_for(reader.get_message(), 0.5)
                self.assertIsNotNone(recv_msg)
                notifier.stop()

        asyncio.run(run_it())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

"""
This example sends every second a messages over the serial interface and also
receives incoming messages.

python3 -m examples.serial_com

Expects two serial ports (/dev/ttyS10 and /dev/ttyS11) connected to each other:
    Linux:
    To connect two ports use socat.
    sudo apt-get install socat
    sudo socat PTY,link=/dev/ttyS10 PTY,link=/dev/ttyS11

    Windows:
    This example was not tested on Windows. To create and connect virtual
    ports on Windows, the following software can be used:
        com0com: http://com0com.sourceforge.net/
"""

import threading
import time

import can


def send_cyclic(bus, msg, stop_event):
    """The loop for sending."""
    print("Start to send a message every 1s")
    start_time = time.time()
    while not stop_event.is_set():
        msg.timestamp = time.time() - start_time
        bus.send(msg)
        print(f"tx: {msg}")
        time.sleep(1)
    print("Stopped sending messages")


def receive(bus, stop_event):
    """The loop for receiving."""
    print("Start receiving messages")
    while not stop_event.is_set():
        rx_msg = bus.recv(1)
        if rx_msg is not None:
            print(f"rx: {rx_msg}")
    print("Stopped receiving messages")


def main():
    """Controls the sender and receiver."""
    with can.Bus(interface="serial", channel="/dev/ttyS10") as server:
        with can.Bus(interface="serial", channel="/dev/ttyS11") as client:
            tx_msg = can.Message(
                arbitration_id=0x01,
                data=[0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88],
            )

            # Thread for sending and receiving messages
            stop_event = threading.Event()
            t_send_cyclic = threading.Thread(
                target=send_cyclic, args=(server, tx_msg, stop_event)
            )
            t_receive = threading.Thread(target=receive, args=(client, stop_event))
            t_receive.start()
            t_send_cyclic.start()

            try:
                while True:
                    time.sleep(0)  # yield
            except KeyboardInterrupt:
                pass  # exit normally

            stop_event.set()
            time.sleep(0.5)

    print("Stopped script")


if __name__ == "__main__":
    main()



"""
python-can documentation build configuration file

This file is execfile()d with the current directory set to its containing dir.
"""

# -- Imports -------------------------------------------------------------------

import ctypes
import os
import sys
from importlib.metadata import version as get_version
from unittest.mock import MagicMock

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath(".."))

from can import ctypesutil  # pylint: disable=wrong-import-position

# -- General configuration -----------------------------------------------------

# pylint: disable=invalid-name

# The version info for the project, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The full version, including alpha/beta/rc tags.
release: str = get_version("python-can")
# The short X.Y version.
version = ".".join(release.split(".")[:2])

# General information about the project.
project = "python-can"

primary_domain = "py"

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.extlinks",
    "sphinx.ext.todo",
    "sphinx.ext.intersphinx",
    "sphinx.ext.coverage",
    "sphinx.ext.viewcode",
    "sphinx.ext.graphviz",
    "sphinxcontrib.programoutput",
    "sphinx_inline_tabs",
    "sphinx_copybutton",
]

# Now, you can use the alias name as a new role, e.g. :issue:`123`.
extlinks = {"issue": ("https://github.com/hardbyte/python-can/issues/%s/", "issue #%s")}

intersphinx_mapping = {"python": ("https://docs.python.org/3/", None)}

# If this is True, todo and todolist produce output, else they produce nothing.
# The default is False.
todo_include_todos = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

graphviz_output_format = "png"  # 'svg'

# The suffix of source filenames.
source_suffix = {".rst": "restructuredtext"}

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = "index"

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
language = "en"

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ["_build"]

# The reST default role (used for this markup: `text`) to use for all documents
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# Include documentation from both the class level and __init__
autoclass_content = "both"

# The default autodoc directive flags
autodoc_default_flags = ["members", "show-inheritance"]

# Keep cached intersphinx inventories indefinitely
intersphinx_cache_limit = -1

# location of typehints
autodoc_typehints = "description"

# disable specific warnings
nitpick_ignore = [
    # Ignore warnings for type aliases. Remove once Sphinx supports PEP613
    ("py:class", "OpenTextModeUpdating"),
    ("py:class", "OpenTextModeWriting"),
    ("py:class", "OpenBinaryModeUpdating"),
    ("py:class", "OpenBinaryModeWriting"),
    ("py:class", "OpenTextModeReading"),
    ("py:class", "OpenBinaryModeReading"),
    ("py:class", "BusConfig"),
    ("py:class", "can.typechecking.BusConfig"),
    ("py:class", "can.typechecking.CanFilter"),
    ("py:class", "can.typechecking.CanFilterExtended"),
    ("py:class", "can.typechecking.AutoDetectedConfig"),
    ("py:class", "can.util.T1"),
    ("py:class", "can.util.T2"),
    ("py:class", "~P1"),
    # intersphinx fails to reference some builtins
    ("py:class", "asyncio.events.AbstractEventLoop"),
    ("py:class", "_thread.lock"),
]

# mock windows specific attributes
autodoc_mock_imports = ["win32com", "pythoncom"]
ctypes.windll = MagicMock()
ctypesutil.HRESULT = ctypes.c_long

# -- Options for HTML output --------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = "furo"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
# html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
# html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
# html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs. This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
# html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
# html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
# html_domain_indices = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, links to the reST sources are added to the pages.
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
# html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
# html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = "python-can"



#!/usr/bin/env python

"""
This example demonstrates how to use async IO with python-can.
"""

import asyncio
from typing import TYPE_CHECKING

import can

if TYPE_CHECKING:
    from can.notifier import MessageRecipient


def print_message(msg: can.Message) -> None:
    """Regular callback function. Can also be a coroutine."""
    print(msg)


async def main() -> None:
    """The main function that runs in the loop."""

    with can.Bus(
        interface="virtual", channel="my_channel_0", receive_own_messages=True
    ) as bus:
        reader = can.AsyncBufferedReader()
        logger = can.Logger("logfile.asc")

        listeners: list[MessageRecipient] = [
            print_message,  # Callback function
            reader,  # AsyncBufferedReader() listener
            logger,  # Regular Listener object
        ]
        # Create Notifier with an explicit loop to use for scheduling of callbacks
        with can.Notifier(bus, listeners, loop=asyncio.get_running_loop()):
            # Start sending first message
            bus.send(can.Message(arbitration_id=0))

            print("Bouncing 10 messages...")
            for _ in range(10):
                # Wait for next message from AsyncBufferedReader
                msg = await reader.get_message()
                # Delay response
                await asyncio.sleep(0.5)
                msg.arbitration_id += 1
                bus.send(msg)

            # Wait for last message to arrive
            await reader.get_message()
            print("Done!")


if __name__ == "__main__":
    asyncio.run(main())



#!/usr/bin/env python

"""
This example exercises the periodic task's multiple message sending capabilities
to send a message containing a counter and a checksum.

Expects a vcan0 interface:

    python3 -m examples.crc

"""

import logging
import time

import can

logging.basicConfig(level=logging.INFO)


def crc_send(bus):
    """
    Sends periodic messages every 1 s with no explicit timeout. Modifies messages
    after 8 seconds, sends for 10 more seconds, then stops.
    """
    msg = can.Message(arbitration_id=0x12345678, data=[1, 2, 3, 4, 5, 6, 7, 0])
    messages = build_crc_msgs(msg)

    print(
        "Starting to send a message with updating counter and checksum every 1 s for 8 s"
    )
    task = bus.send_periodic(messages, 1)
    assert isinstance(task, can.CyclicSendTaskABC)
    time.sleep(8)

    msg = can.Message(arbitration_id=0x12345678, data=[8, 9, 10, 11, 12, 13, 14, 0])
    messages = build_crc_msgs(msg)

    print("Sending modified message data every 1 s for 10 s")
    task.modify_data(messages)
    time.sleep(10)
    task.stop()
    print("stopped cyclic send")


def build_crc_msgs(msg):
    """
    Using the input message as base, create 16 messages with SAE J1939 SPN 3189 counters
    and SPN 3188 checksums placed in the final byte.
    """
    messages = []

    for counter in range(16):
        checksum = compute_xbr_checksum(msg, counter)
        msg.data[7] = counter + (checksum << 4)
        messages.append(
            can.Message(arbitration_id=msg.arbitration_id, data=msg.data[:])
        )

    return messages


def compute_xbr_checksum(message, counter):
    """
    Computes an XBR checksum per SAE J1939 SPN 3188.
    """
    checksum = sum(message.data[:7])
    checksum += sum(message.arbitration_id.to_bytes(length=4, byteorder="big"))
    checksum += counter & 0x0F
    xbr_checksum = ((checksum >> 4) + checksum) & 0x0F

    return xbr_checksum


if __name__ == "__main__":
    for interface, channel in [("socketcan", "vcan0")]:
        print(f"Carrying out crc test with {interface} interface")

        with can.Bus(interface=interface, channel=channel, bitrate=500000) as BUS:
            crc_send(BUS)

    time.sleep(2)



#!/usr/bin/env python

"""
This example exercises the periodic sending capabilities.

Expects a vcan0 interface:

    python3 -m examples.cyclic

"""

import logging
import time

import can

logging.basicConfig(level=logging.INFO)


def simple_periodic_send(bus):
    """
    Sends a message every 20ms with no explicit timeout
    Sleeps for 2 seconds then stops the task.
    """
    print("Starting to send a message every 200ms for 2s")
    msg = can.Message(
        arbitration_id=0x123, data=[1, 2, 3, 4, 5, 6], is_extended_id=False
    )
    task = bus.send_periodic(msg, 0.20)
    assert isinstance(task, can.CyclicSendTaskABC)
    time.sleep(2)
    task.stop()
    print("stopped cyclic send")


def limited_periodic_send(bus):
    """Send using LimitedDurationCyclicSendTaskABC."""
    print("Starting to send a message every 200ms for 1s")
    msg = can.Message(
        arbitration_id=0x12345678, data=[0, 0, 0, 0, 0, 0], is_extended_id=True
    )
    task = bus.send_periodic(msg, 0.20, 1, store_task=False)
    if not isinstance(task, can.LimitedDurationCyclicSendTaskABC):
        print("This interface doesn't seem to support LimitedDurationCyclicSendTaskABC")
        task.stop()
        return

    time.sleep(2)
    print("Cyclic send should have stopped as duration expired")
    # Note the (finished) task will still be tracked by the Bus
    # unless we pass `store_task=False` to bus.send_periodic
    # alternatively calling stop removes the task from the bus
    # task.stop()


def test_periodic_send_with_modifying_data(bus):
    """Send using ModifiableCyclicTaskABC."""
    print("Starting to send a message every 200ms. Initial data is four consecutive 1s")
    msg = can.Message(arbitration_id=0x0CF02200, data=[1, 1, 1, 1])
    task = bus.send_periodic(msg, 0.20)
    if not isinstance(task, can.ModifiableCyclicTaskABC):
        print("This interface doesn't seem to support modification")
        task.stop()
        return
    time.sleep(2)
    print("Changing data of running task to begin with 99")
    msg.data[0] = 0x99
    task.modify_data(msg)
    time.sleep(2)

    task.stop()
    print("stopped cyclic send")
    print("Changing data of stopped task to single ff byte")
    msg.data = bytearray([0xFF])
    msg.dlc = 1
    task.modify_data(msg)
    time.sleep(1)
    print("starting again")
    task.start()
    time.sleep(1)
    task.stop()
    print("done")


# Will have to consider how to expose items like this. The socketcan
# interfaces will continue to support it... but the top level api won't.
# def test_dual_rate_periodic_send():
#     """Send a message 10 times at 1ms intervals, then continue to send every 500ms"""
#     msg = can.Message(arbitration_id=0x123, data=[0, 1, 2, 3, 4, 5])
#     print("Creating cyclic task to send message 10 times at 1ms, then every 500ms")
#     task = can.interface.MultiRateCyclicSendTask('vcan0', msg, 10, 0.001, 0.50)
#     time.sleep(2)
#
#     print("Changing data[0] = 0x42")
#     msg.data[0] = 0x42
#     task.modify_data(msg)
#     time.sleep(2)
#
#     task.stop()
#     print("stopped cyclic send")
#
#     time.sleep(2)
#
#     task.start()
#     print("starting again")
#     time.sleep(2)
#     task.stop()
#     print("done")


def main():
    """Test different cyclic sending tasks."""
    reset_msg = can.Message(
        arbitration_id=0x00, data=[0, 0, 0, 0, 0, 0], is_extended_id=False
    )

    # this uses the default configuration (for example from environment variables, or a
    # config file) see https://python-can.readthedocs.io/en/stable/configuration.html
    with can.Bus() as bus:
        bus.send(reset_msg)

        simple_periodic_send(bus)

        bus.send(reset_msg)

        limited_periodic_send(bus)

        test_periodic_send_with_modifying_data(bus)

        # print("Carrying out multirate cyclic test for {} interface".format(interface))
        # can.rc['interface'] = interface
        # test_dual_rate_periodic_send()

    time.sleep(2)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import unittest
import can
from can.interfaces.socketcand import socketcand


class TestConvertAsciiMessageToCanMessage(unittest.TestCase):
    def test_valid_frame_message(self):
        # Example: < frame 123 1680000000.0 01020304 >
        ascii_msg = "< frame 123 1680000000.0 01020304 >"
        msg = socketcand.convert_ascii_message_to_can_message(ascii_msg)
        self.assertIsInstance(msg, can.Message)
        self.assertEqual(msg.arbitration_id, 0x123)
        self.assertEqual(msg.timestamp, 1680000000.0)
        self.assertEqual(msg.data, bytearray([1, 2, 3, 4]))
        self.assertEqual(msg.dlc, 4)
        self.assertFalse(msg.is_extended_id)
        self.assertTrue(msg.is_rx)

    def test_valid_error_message(self):
        # Example: < error 1ABCDEF0 1680000001.0 >
        ascii_msg = "< error 1ABCDEF0 1680000001.0 >"
        msg = socketcand.convert_ascii_message_to_can_message(ascii_msg)
        self.assertIsInstance(msg, can.Message)
        self.assertEqual(msg.arbitration_id, 0x1ABCDEF0)
        self.assertEqual(msg.timestamp, 1680000001.0)
        self.assertEqual(msg.data, bytearray([0]))
        self.assertEqual(msg.dlc, 1)
        self.assertTrue(msg.is_extended_id)
        self.assertTrue(msg.is_error_frame)
        self.assertTrue(msg.is_rx)

    def test_invalid_message(self):
        ascii_msg = "< unknown 123 0.0 >"
        msg = socketcand.convert_ascii_message_to_can_message(ascii_msg)
        self.assertIsNone(msg)

    def test_missing_ending_character(self):
        ascii_msg = "< frame 123 1680000000.0 01020304"
        msg = socketcand.convert_ascii_message_to_can_message(ascii_msg)
        self.assertIsNone(msg)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

""" """

import ctypes
import time
import unittest
from unittest.mock import Mock

import pytest

import can
from can.interfaces.kvaser import canlib, constants


class KvaserTest(unittest.TestCase):
    def setUp(self):
        canlib.canGetNumberOfChannels = KvaserTest.canGetNumberOfChannels
        canlib.canOpenChannel = Mock(return_value=0)
        canlib.canIoCtl = Mock(return_value=0)
        canlib.canIoCtlInit = Mock(return_value=0)
        canlib.kvReadTimer = Mock()
        canlib.canSetBusParamsC200 = Mock()
        canlib.canSetBusParams = Mock()
        canlib.canSetBusParamsFd = Mock()
        canlib.canBusOn = Mock()
        canlib.canBusOff = Mock()
        canlib.canClose = Mock()
        canlib.canSetBusOutputControl = Mock()
        canlib.canGetChannelData = Mock()
        canlib.canSetAcceptanceFilter = Mock()
        canlib.canWriteSync = Mock()
        canlib.canWrite = self.canWrite
        canlib.canReadWait = self.canReadWait
        canlib.canGetBusStatistics = Mock()
        canlib.canRequestBusStatistics = Mock()

        self.msg = {}
        self.msg_in_cue = None
        self.bus = can.Bus(channel=0, interface="kvaser")

    def tearDown(self):
        if self.bus:
            self.bus.shutdown()
            self.bus = None

    def test_bus_creation(self):
        self.assertIsInstance(self.bus, canlib.KvaserBus)
        self.assertEqual(self.bus.protocol, can.CanProtocol.CAN_20)
        self.assertTrue(canlib.canOpenChannel.called)
        self.assertTrue(canlib.canBusOn.called)

    def test_bus_creation_illegal_channel_name(self):
        # Test if the bus constructor is able to deal with non-ASCII characters
        def canGetChannelDataMock(
            channel: ctypes.c_int,
            param: ctypes.c_int,
            buf: ctypes.c_void_p,
            bufsize: ctypes.c_size_t,
        ):
            if param == constants.canCHANNELDATA_DEVDESCR_ASCII:
                buf_char_ptr = ctypes.cast(buf, ctypes.POINTER(ctypes.c_char))
                for i, char in enumerate(b"hello\x7a\xcb"):
                    buf_char_ptr[i] = char

        canlib.canGetChannelData = canGetChannelDataMock
        bus = can.Bus(channel=0, interface="kvaser")

        self.assertTrue(bus.channel_info.startswith("hello"))

        bus.shutdown()

    def test_bus_shutdown(self):
        self.bus.shutdown()
        self.assertTrue(canlib.canBusOff.called)
        self.assertTrue(canlib.canClose.called)

    def test_filter_setup(self):
        # No filter in constructor
        expected_args = [
            ((0, 0, 0, 0),),  # Disable filtering STD on read handle
            ((0, 0, 0, 1),),  # Disable filtering EXT on read handle
            ((0, 0, 0, 0),),  # Disable filtering STD on write handle
            ((0, 0, 0, 1),),  # Disable filtering EXT on write handle
        ]
        self.assertEqual(canlib.canSetAcceptanceFilter.call_args_list, expected_args)

        # One filter, will be handled by canlib
        canlib.canSetAcceptanceFilter.reset_mock()
        self.bus.set_filters([{"can_id": 0x8, "can_mask": 0xFF, "extended": True}])
        expected_args = [
            ((0, 0x8, 0xFF, 1),),  # Enable filtering EXT on read handle
            ((0, 0x8, 0xFF, 1),),  # Enable filtering EXT on write handle
        ]
        self.assertEqual(canlib.canSetAcceptanceFilter.call_args_list, expected_args)

        # Multiple filters, will be handled in Python
        canlib.canSetAcceptanceFilter.reset_mock()
        multiple_filters = [
            {"can_id": 0x8, "can_mask": 0xFF},
            {"can_id": 0x9, "can_mask": 0xFF},
        ]
        self.bus.set_filters(multiple_filters)
        expected_args = [
            ((0, 0, 0, 0),),  # Disable filtering STD on read handle
            ((0, 0, 0, 1),),  # Disable filtering EXT on read handle
            ((0, 0, 0, 0),),  # Disable filtering STD on write handle
            ((0, 0, 0, 1),),  # Disable filtering EXT on write handle
        ]
        self.assertEqual(canlib.canSetAcceptanceFilter.call_args_list, expected_args)

    def test_send_extended(self):
        msg = can.Message(
            arbitration_id=0xC0FFEE, data=[0, 25, 0, 1, 3, 1, 4], is_extended_id=True
        )

        self.bus.send(msg)

        self.assertEqual(self.msg["arb_id"], 0xC0FFEE)
        self.assertEqual(self.msg["dlc"], 7)
        self.assertEqual(self.msg["flags"], constants.canMSG_EXT)
        self.assertSequenceEqual(self.msg["data"], [0, 25, 0, 1, 3, 1, 4])

    def test_send_standard(self):
        msg = can.Message(arbitration_id=0x321, data=[50, 51], is_extended_id=False)

        self.bus.send(msg)

        self.assertEqual(self.msg["arb_id"], 0x321)
        self.assertEqual(self.msg["dlc"], 2)
        self.assertEqual(self.msg["flags"], constants.canMSG_STD)
        self.assertSequenceEqual(self.msg["data"], [50, 51])

    @pytest.mark.timeout(3.0)
    def test_recv_no_message(self):
        self.assertEqual(self.bus.recv(timeout=0.5), None)

    def test_recv_extended(self):
        self.msg_in_cue = can.Message(
            arbitration_id=0xC0FFEF, data=[1, 2, 3, 4, 5, 6, 7, 8], is_extended_id=True
        )

        now = time.time()
        msg = self.bus.recv()
        self.assertEqual(msg.arbitration_id, 0xC0FFEF)
        self.assertEqual(msg.dlc, 8)
        self.assertEqual(msg.is_extended_id, True)
        self.assertSequenceEqual(msg.data, self.msg_in_cue.data)
        self.assertTrue(now - 1 < msg.timestamp < now + 1)

    def test_recv_standard(self):
        self.msg_in_cue = can.Message(
            arbitration_id=0x123, data=[100, 101], is_extended_id=False
        )

        msg = self.bus.recv()
        self.assertEqual(msg.arbitration_id, 0x123)
        self.assertEqual(msg.dlc, 2)
        self.assertEqual(msg.is_extended_id, False)
        self.assertSequenceEqual(msg.data, [100, 101])

    def test_available_configs(self):
        configs = canlib.KvaserBus._detect_available_configs()
        expected = [
            {
                "interface": "kvaser",
                "channel": 0,
                "dongle_channel": 1,
                "device_name": "",
                "serial": 0,
            },
            {
                "interface": "kvaser",
                "channel": 1,
                "dongle_channel": 1,
                "device_name": "",
                "serial": 0,
            },
        ]
        self.assertListEqual(configs, expected)

    def test_canfd_default_data_bitrate(self):
        canlib.canSetBusParams.reset_mock()
        canlib.canSetBusParamsFd.reset_mock()
        bus = can.Bus(channel=0, interface="kvaser", fd=True)
        self.assertEqual(bus.protocol, can.CanProtocol.CAN_FD)
        canlib.canSetBusParams.assert_called_once_with(
            0, constants.canFD_BITRATE_500K_80P, 0, 0, 0, 0, 0
        )
        canlib.canSetBusParamsFd.assert_called_once_with(
            0, constants.canFD_BITRATE_500K_80P, 0, 0, 0
        )

    def test_can_timing(self):
        canlib.canSetBusParams.reset_mock()
        canlib.canSetBusParamsFd.reset_mock()
        timing = can.BitTiming.from_bitrate_and_segments(
            f_clock=16_000_000,
            bitrate=125_000,
            tseg1=13,
            tseg2=2,
            sjw=1,
        )
        can.Bus(channel=0, interface="kvaser", timing=timing)
        canlib.canSetBusParamsC200.assert_called_once_with(0, timing.btr0, timing.btr1)

    def test_canfd_timing(self):
        canlib.canSetBusParams.reset_mock()
        canlib.canSetBusParamsFd.reset_mock()
        canlib.canOpenChannel.reset_mock()
        timing = can.BitTimingFd.from_bitrate_and_segments(
            f_clock=80_000_000,
            nom_bitrate=500_000,
            nom_tseg1=68,
            nom_tseg2=11,
            nom_sjw=10,
            data_bitrate=2_000_000,
            data_tseg1=10,
            data_tseg2=9,
            data_sjw=8,
        )
        can.Bus(channel=0, interface="kvaser", timing=timing)
        canlib.canSetBusParams.assert_called_once_with(0, 500_000, 68, 11, 10, 1, 0)
        canlib.canSetBusParamsFd.assert_called_once_with(0, 2_000_000, 10, 9, 8)
        canlib.canOpenChannel.assert_called_with(
            0, constants.canOPEN_CAN_FD | constants.canOPEN_ACCEPT_VIRTUAL
        )

    def test_canfd_non_iso(self):
        canlib.canSetBusParams.reset_mock()
        canlib.canSetBusParamsFd.reset_mock()
        canlib.canOpenChannel.reset_mock()
        timing = can.BitTimingFd.from_bitrate_and_segments(
            f_clock=80_000_000,
            nom_bitrate=500_000,
            nom_tseg1=68,
            nom_tseg2=11,
            nom_sjw=10,
            data_bitrate=2_000_000,
            data_tseg1=10,
            data_tseg2=9,
            data_sjw=8,
        )
        bus = can.Bus(channel=0, interface="kvaser", timing=timing, fd_non_iso=True)
        self.assertEqual(bus.protocol, can.CanProtocol.CAN_FD_NON_ISO)
        canlib.canSetBusParams.assert_called_once_with(0, 500_000, 68, 11, 10, 1, 0)
        canlib.canSetBusParamsFd.assert_called_once_with(0, 2_000_000, 10, 9, 8)
        canlib.canOpenChannel.assert_called_with(
            0, constants.canOPEN_CAN_FD_NONISO | constants.canOPEN_ACCEPT_VIRTUAL
        )

    def test_canfd_nondefault_data_bitrate(self):
        canlib.canSetBusParams.reset_mock()
        canlib.canSetBusParamsFd.reset_mock()
        data_bitrate = 2000000
        bus = can.Bus(channel=0, interface="kvaser", fd=True, data_bitrate=data_bitrate)
        self.assertEqual(bus.protocol, can.CanProtocol.CAN_FD)
        bitrate_constant = canlib.BITRATE_FD[data_bitrate]
        canlib.canSetBusParams.assert_called_once_with(
            0, constants.canFD_BITRATE_500K_80P, 0, 0, 0, 0, 0
        )
        canlib.canSetBusParamsFd.assert_called_once_with(0, bitrate_constant, 0, 0, 0)

    def test_canfd_custom_data_bitrate(self):
        canlib.canSetBusParams.reset_mock()
        canlib.canSetBusParamsFd.reset_mock()
        data_bitrate = 123456
        can.Bus(channel=0, interface="kvaser", fd=True, data_bitrate=data_bitrate)
        canlib.canSetBusParams.assert_called_once_with(
            0, constants.canFD_BITRATE_500K_80P, 0, 0, 0, 0, 0
        )
        canlib.canSetBusParamsFd.assert_called_once_with(0, data_bitrate, 0, 0, 0)

    def test_bus_get_stats(self):
        stats = self.bus.get_stats()
        self.assertTrue(canlib.canRequestBusStatistics.called)
        self.assertTrue(canlib.canGetBusStatistics.called)
        self.assertIsInstance(stats, canlib.structures.BusStatistics)

    def test_bus_no_init_access(self):
        canlib.canOpenChannel.reset_mock()
        bus = can.Bus(interface="kvaser", channel=0, no_init_access=True)

        self.assertGreater(canlib.canOpenChannel.call_count, 0)
        for call in canlib.canOpenChannel.call_args_list:
            self.assertEqual(
                call[0][1] & constants.canOPEN_NO_INIT_ACCESS,
                constants.canOPEN_NO_INIT_ACCESS,
            )

        bus.shutdown()

    @staticmethod
    def canGetNumberOfChannels(count):
        count._obj.value = 2

    def canWrite(self, handle, arb_id, buf, dlc, flags):
        self.msg["arb_id"] = arb_id
        self.msg["dlc"] = dlc
        self.msg["flags"] = flags
        self.msg["data"] = bytearray(buf._obj)

    def canReadWait(self, handle, arb_id, data, dlc, flags, timestamp, timeout):
        if not self.msg_in_cue:
            return constants.canERR_NOMSG

        arb_id._obj.value = self.msg_in_cue.arbitration_id
        dlc._obj.value = self.msg_in_cue.dlc
        data._obj.raw = self.msg_in_cue.data
        flags_temp = 0
        if self.msg_in_cue.is_extended_id:
            flags_temp |= constants.canMSG_EXT
        else:
            flags_temp |= constants.canMSG_STD
        if self.msg_in_cue.is_remote_frame:
            flags_temp |= constants.canMSG_RTR
        if self.msg_in_cue.is_error_frame:
            flags_temp |= constants.canMSG_ERROR_FRAME
        flags._obj.value = flags_temp
        timestamp._obj.value = 0

        return constants.canOK


if __name__ == "__main__":
    unittest.main()
